/*
Copyright 2011 Daniel Morton. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice, this list of
      conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright notice, this list
      of conditions and the following disclaimer in the documentation and/or other materials
      provided with the distribution.

THIS SOFTWARE IS PROVIDED BY Daniel Morton ''AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those of the
authors and should not be interpreted as representing official policies, either expressed
or implied, of Daniel Morton.
*/

package ca.quadrilateral.wallgenerator;

import static info.clearthought.layout.TableLayoutConstants.FILL;
import static info.clearthought.layout.TableLayoutConstants.PREFERRED;
import info.clearthought.layout.TableLayout;

import java.awt.Dimension;
import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;

import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang.StringUtils;

public class FileChooserPanel extends JPanel {
	private static final long serialVersionUID = 1L;
	
	private double[][] layoutDescriptor = null;
    private TableLayout layout = null;

    private String sourceImageProperties = "";

    private final Collection<String> validExtensions = Arrays.asList("JPG", "JPEG", "BMP", "GIF", "PNG");

    private final JTextField sourceFileNameTextField = new JTextField();
    private final JTextField targetDirectoryTextField = new JTextField();
    private final JTextField outputFileNameBaseTextField = new JTextField();

    private final JButton sourceFileNameButton = new JButton("Browse");
    private final JButton targetDirectoryNameButton = new JButton("Browse");
    private final JButton generateBaseFileNameButton = new JButton("Generate");

    private final ImagePreviewPanel previewPanel = new ImagePreviewPanel(112, 134);

    private final JFileChooser sourceImageFileChooser = new JFileChooser();
    private final JFileChooser targetDirectoryFileChooser = new JFileChooser();
    
    //private final JPanel sourceImageConfigPanel = new JPanel();
    
    //private final JLabel interpolationLabel = new JLabel("Interpolation:");
    //private final JLabel placementLabel = new JLabel("Placement:");
    
    //private final JRadioButton scaleRadioButton = new JRadioButton("Scale", true);
    //private final JRadioButton positionRadioButton = new JRadioButton("Position");
    
    //private final ButtonGroup sourceImageManipulationButtonGroup = new ButtonGroup();
    
    //private final JComboBox interpolationCombo = new JComboBox();
    //private final JComboBox positionCombo = new JComboBox();
    
    private String currentResolution = "";
    private String currentOutputFormat = "";

    private Collection<ActionListener> changeGenerationSettingsListeners = new ArrayList<ActionListener>();

//    public boolean useScaling() {
//    	return scaleRadioButton.isSelected();
//    }
//    
//    public Object getInterpolationRenderingHint() {
//    	return ((InterpolationContainer)interpolationCombo.getSelectedItem()).getRenderingHint();
//    }
//    
//    public String getPosition() {
//    	return (String)positionCombo.getSelectedItem();
//    }
    
    
    public FileChooserPanel() {
        super();
        this.setLayout(setupLayout());
        this.setPreferredSize(new Dimension(1, 170));
        this.setBorder(BorderFactory.createEtchedBorder());
        this.setupComponents();
        this.placeComponents();
        this.attachListeners();
        this.setupSourceImageFileChooser();
        this.setupTargetDirectoryFileChooser();
        //this.setupSourceImageConfigPanel();
    }
    
    public void setCurrentResolution(String currentResolution) {
    	this.currentResolution = currentResolution;
    }
    
    public void setCurrentOutputFormat(String currentOutputFormat) {
    	this.currentOutputFormat = currentOutputFormat;
    }

    public void addChangeGenerationSettingsListener(ActionListener actionListener) {
        this.changeGenerationSettingsListeners.add(actionListener);
    }

    private void fireChangeGenerationSettingsListeners(ActionEvent event) {
        for(ActionListener listener : this.changeGenerationSettingsListeners) {
            listener.actionPerformed(event);
        }
    }

//    private void setupSourceImageConfigPanel() {
//    	final double[][] layoutDescriptor = new double[][] {
//    			{5, TableLayout.PREFERRED, 15, TableLayout.PREFERRED, 5, TableLayout.PREFERRED, 5}, 
//    			{5, TableLayout.PREFERRED, 5, TableLayout.PREFERRED, 5}
//    		};
//    	final TableLayout layout = new TableLayout(layoutDescriptor);
//    	sourceImageConfigPanel.setLayout(layout);
//    	
//    	sourceImageConfigPanel.add(scaleRadioButton, "1, 1");
//    	sourceImageConfigPanel.add(positionRadioButton, "1, 3");
//    	sourceImageConfigPanel.add(interpolationLabel, "3, 1");
//    	sourceImageConfigPanel.add(placementLabel, "3, 3");
//    	sourceImageConfigPanel.add(interpolationCombo, "5, 1");
//    	sourceImageConfigPanel.add(positionCombo, "5, 3");
//    	
//    	sourceImageManipulationButtonGroup.add(scaleRadioButton);
//    	sourceImageManipulationButtonGroup.add(positionRadioButton);
//    	
//    	DefaultComboBoxModel interpolationModel = (DefaultComboBoxModel)interpolationCombo.getModel();
//    	interpolationModel.addElement(new InterpolationContainer("Nearest Neighbour", RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR));
//    	interpolationModel.addElement(new InterpolationContainer("Bilinear", RenderingHints.VALUE_INTERPOLATION_BILINEAR));
//    	interpolationModel.addElement(new InterpolationContainer("Bicubic", RenderingHints.VALUE_INTERPOLATION_BICUBIC));
//    	interpolationModel.addElement(new InterpolationContainer("Trilinear", 255));
//    	
//    	DefaultComboBoxModel positionModel = (DefaultComboBoxModel)positionCombo.getModel();
//    	positionModel.addElement("CENTER");
//    	positionModel.addElement("N");
//    	positionModel.addElement("NE");
//    	positionModel.addElement("E");
//    	positionModel.addElement("SE");
//    	positionModel.addElement("S");
//    	positionModel.addElement("SW");
//    	positionModel.addElement("W");
//    	positionModel.addElement("NW");
//    	
//    	placementLabel.setEnabled(false);
//    	positionCombo.setEnabled(false);
//    	
//    	scaleRadioButton.addActionListener(sourceConfigurationActionListener);
//    	positionRadioButton.addActionListener(sourceConfigurationActionListener);
//
//    	this.add(new JLabel("Source Manipulation:"), "1, 7");
//    	this.add(this.sourceImageConfigPanel, "3, 7");
//    	
//    }
    
    private final void setupSourceImageFileChooser() {
        final JFileChooser fileChooser = sourceImageFileChooser;
        fileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
        fileChooser.setDialogTitle("Select Source File");
        fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        fileChooser.setFileFilter(new FileFilter() {
            @Override
            public boolean accept(File f) {
                final String extension = FilenameUtils.getExtension(f.getAbsolutePath()).toUpperCase();
                return f.isDirectory() || validExtensions.contains(extension);
            }

            @Override
            public String getDescription() {
                return "Supported Image Formats (*.jpg, *.jpeg, *.gif, *.bmp, *.png)";
            }
        });

    }

    private final void setupTargetDirectoryFileChooser() {
        final JFileChooser fileChooser = targetDirectoryFileChooser;
        fileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
        fileChooser.setDialogTitle("Select Output Directory");
        fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
    }

    private final void attachListeners() {
        this.sourceFileNameButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (sourceImageFileChooser.showOpenDialog(FileChooserPanel.this) == JFileChooser.APPROVE_OPTION) {
                    sourceFileNameTextField.setText(sourceImageFileChooser.getSelectedFile().getAbsolutePath());
                    try {
                        loadSourcePreview();
                    } catch (Exception ex) {
                        JOptionPane.showMessageDialog(FileChooserPanel.this, "Unable to load image preview!", "Load Error", JOptionPane.ERROR_MESSAGE);
                    }
                }
            }
        });

        this.targetDirectoryNameButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (targetDirectoryFileChooser.showOpenDialog(FileChooserPanel.this) == JFileChooser.APPROVE_OPTION) {
                    targetDirectoryTextField.setText(targetDirectoryFileChooser.getSelectedFile().getAbsolutePath());
                }
            }
        });

        this.generateBaseFileNameButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                final String sourceFileName = sourceFileNameTextField.getText();
                
                
                
                if (!StringUtils.isBlank(sourceFileName)) {
                   outputFileNameBaseTextField.setText(MessageFormat.format("wall_{0}_{1}_{2}", currentResolution.replace(' ', '_'), currentOutputFormat.replace(' ', '_'), FilenameUtils.getBaseName(sourceFileName).replace(' ', '_')));
                } else {
                    JOptionPane.showMessageDialog(FileChooserPanel.this, "Can not generate base file name when no source image is selected!", "No Source Image Selected!", JOptionPane.WARNING_MESSAGE);
                }
            }
        });

        this.sourceFileNameButton.addActionListener(new ChangeGenerationSettingActionListener());
    }


    private final void setupComponents() {
        this.previewPanel.setPreferredSize(new Dimension(112, 134));
        this.sourceFileNameTextField.setEditable(false);
        this.targetDirectoryTextField.setEditable(false);
        this.targetDirectoryTextField.setText(System.getProperty("user.dir"));
    }

    private final void placeComponents() {
        this.add(new JLabel("Source File:"), "1, 1");
        this.add(new JLabel("Destination Dir:"), "1, 3");
        this.add(new JLabel("Base Output File Name:"), "1, 5");
        this.add(new JLabel("Preview"), "7, 1, c, f");

        this.add(sourceFileNameTextField, "3, 1, f, f");
        this.add(targetDirectoryTextField, "3, 3, f, f");
        this.add(outputFileNameBaseTextField, "3, 5, f, f");

        this.add(sourceFileNameButton, "5, 1, f, f");
        this.add(targetDirectoryNameButton, "5, 3, f, f");
        this.add(generateBaseFileNameButton, "5, 5, f, f");

        this.add(this.previewPanel, "7, 3, 7, 9");

    }

    private final LayoutManager setupLayout() {
        this.layoutDescriptor = new double[][] {
                {5, PREFERRED, 5, FILL, 5, PREFERRED, 5, 112, 5},
                {3, PREFERRED, 3, PREFERRED, 3, PREFERRED, 3, PREFERRED, 3, FILL, 3}
        };

        this.layout = new TableLayout(layoutDescriptor);
        return this.layout;
    }

    private void loadSourcePreview() throws Exception {
        final BufferedImage sourceImage = ImageIO.read(new File(sourceFileNameTextField.getText()));
        previewPanel.setImage(sourceImage);
        previewPanel.repaint();

        sourceImageProperties = MessageFormat.format("{0}x{1}", sourceImage.getWidth(), sourceImage.getHeight());

        previewPanel.setToolTipText(sourceImageProperties);
    }


    public File getSourceFile() {
        final String sourceFileNameText = sourceFileNameTextField.getText();
        if (StringUtils.isBlank(sourceFileNameText)) {
            return null;
        }

        final File file = new File(sourceFileNameTextField.getText());

        if (file.isFile() && file.canRead()) {
            return file;
        } else {
            return null;
        }
    }

    public File getTargetDirectory() {
        final String targetDirectoryText = targetDirectoryTextField.getText();
        if (StringUtils.isBlank(targetDirectoryText)) {
            return null;
        }

        final File file = new File(targetDirectoryTextField.getText());
        if (file.isDirectory()) {
            return file;
        } else {
            return null;
        }
    }

    public String outputFileNameBase() {
        return this.outputFileNameBaseTextField.getText();
    }

//    private ActionListener sourceConfigurationActionListener = new ActionListener() {
//		@Override
//		public void actionPerformed(ActionEvent e) {
//			boolean isScale = scaleRadioButton.isSelected();
//			
//			interpolationLabel.setEnabled(isScale);
//			interpolationCombo.setEnabled(isScale);
//			
//			placementLabel.setEnabled(!isScale);
//			positionCombo.setEnabled(!isScale);								
//		}
//    };
    
    public class ChangeGenerationSettingActionListener implements ActionListener {
        @Override
        public void actionPerformed(ActionEvent event) {
            fireChangeGenerationSettingsListeners(event);
        }
    }
        
    public class InterpolationContainer {
    	private final String label;
    	private final Object renderingHint;
    	
    	public InterpolationContainer(String label, Object renderingHint) {
    		this.label = label;
    		this.renderingHint = renderingHint;
    	}
    	
    	@Override
    	public String toString() {
    		return this.label;
    	}
    	
    	public Object getRenderingHint() {
    		return this.renderingHint;
    	}
    }

}
