/*
Copyright 2011 Daniel Morton. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice, this list of
      conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright notice, this list
      of conditions and the following disclaimer in the documentation and/or other materials
      provided with the distribution.

THIS SOFTWARE IS PROVIDED BY Daniel Morton ''AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those of the
authors and should not be interpreted as representing official policies, either expressed
or implied, of Daniel Morton.
*/

package ca.quadrilateral.wallgenerator;

import info.clearthought.layout.TableLayout;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.DefaultComboBoxModel;
import javax.swing.InputVerifier;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.border.BevelBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.mortennobel.imagescaling.AdvancedResizeOp.UnsharpenMask;

public class OptionPanel extends JPanel {
	private static final Logger log = LoggerFactory.getLogger(OptionPanel.class);
    private static final long serialVersionUID = 1L;

    private double[][] layoutDescriptor = null;
    private TableLayout layout = null;

    private JRadioButton pngFileTypeRadio = new JRadioButton("PNG", true);
    private JRadioButton gifFileTypeRadio = new JRadioButton("GIF");
    private JRadioButton bmpFileTypeRadio = new JRadioButton("BMP");

    private Collection<JRadioButton> resolutionButtons = new ArrayList<JRadioButton>();
    private Collection<JRadioButton> outputFileTypeButtons = new ArrayList<JRadioButton>();
    private Collection<JRadioButton> outputFormatButtons = new ArrayList<JRadioButton>();

    private ButtonGroup resolutionButtonGroup = new ButtonGroup();
    private ButtonGroup outputFormatButtonGroup = new ButtonGroup();
    private ButtonGroup outputFileTypeButtonGroup = new ButtonGroup();

    private JButton previewWallsButton = new JButton("Preview Walls");
    private JButton generateWallsButton = new JButton("Generate Walls");

    private JSpinner nearByDarkeningSpinner = new JSpinner(new SpinnerNumberModel(0, 0, 1, 0.01));
    private JSpinner farAwayDarkeningSpinner = new JSpinner(new SpinnerNumberModel(0, 0, 1, 0.01));

    private JButton changeBackgroundColorButton = new JButton("Change");
    private JButton changeTransparencyColorButton = new JButton("Change");
    private JButton changeTextColorButton = new JButton("Change");
    private JButton changeBorderColorButton = new JButton("Change");
    
    private JPanel selectedBackgroundColorPanel = new JPanel();
    private JPanel selectedTransparencyColorPanel = new JPanel();
    private JPanel selectedBorderColorPanel = new JPanel();
    private JPanel selectedTextColorPanel = new JPanel();
    
    private JRadioButton useTransparentColorRadioButton = new JRadioButton();
    private JRadioButton useTransparencyRadioButton = new JRadioButton("Transparent");
    
    private ButtonGroup transparencyRadioButtonGroup = new ButtonGroup();
    
    private Collection<ActionListener> previewWallsActionListeners = new ArrayList<ActionListener>();
    private Collection<ActionListener> generateWallsActionListeners = new ArrayList<ActionListener>();
    private Collection<ActionListener> changeGenerationSettingsListeners = new ArrayList<ActionListener>();

    private JComboBox scalingMethodComboBox = new JComboBox();
    private JComboBox unsharpMaskComboBox = new JComboBox();
    private JComboBox processOrderComboBox = new JComboBox(new Object[] {"Unsharp then Soften", "Soften then Unsharp"});
    private JSpinner softeningSpinner = new JSpinner(new SpinnerNumberModel(0, 0, 1, 0.01));
    
    
    
    private JPanel resolutionPanel = null;
    private JPanel outputFormatPanel = null;
    private JPanel outputFileTypePanel = null;
    private JPanel colorPanel = null;
    private JPanel processingPanel = null;
    private JPanel buttonPanel = null;

    public OptionPanel() {
        super();
        this.setupContainerPanels();

        this.setLayout(setupLayout());
        this.setBorder(BorderFactory.createEtchedBorder());
        this.setupComponents();
        this.placeComponents();
        this.attachListeners();
    }

    private void setupContainerPanels() {
        setupResolutionPanel();
        setupOutputFormatPanel();
        setupOutputFileTypePanel();
        setupProcessingPanel();
        setupColorPanel();
        setupButtonPanel();
    }
    
    private void setupButtonPanel() {
    	final double[][] layoutDescriptor = new double[][] {{TableLayout.FILL, TableLayout.PREFERRED, 5, TableLayout.PREFERRED, 3},{3, TableLayout.PREFERRED, 3}};
    	final TableLayout layout = new TableLayout(layoutDescriptor);
    	this.buttonPanel = new JPanel(layout);
    	
    	this.buttonPanel.add(this.previewWallsButton, "1, 1");
    	this.buttonPanel.add(this.generateWallsButton, "3, 1");
    }
    
    private void setupColorPanel() {
    	colorPanel = new JPanel(new BorderLayout());
    	colorPanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
    	
    	final double[][] colorPanelLayoutDescriptor = new double[][] {
    			{3, TableLayout.PREFERRED, 3, TableLayout.PREFERRED, 3, 50, 3, TableLayout.PREFERRED, 3}, 
    			{3, TableLayout.PREFERRED, 3, TableLayout.PREFERRED, 3, TableLayout.PREFERRED, 3, TableLayout.PREFERRED, 3, TableLayout.PREFERRED, 3}
    		};
    	
    	final TableLayout colorPanelLayout = new TableLayout(colorPanelLayoutDescriptor);
    	
    	final JPanel optionSubPanel = new JPanel(colorPanelLayout);
    	
    	final JLabel titleLabel = new JLabel("Colors");
    	titleLabel.setHorizontalAlignment(SwingConstants.CENTER);
    	
    	colorPanel.add(titleLabel, BorderLayout.NORTH);
    	colorPanel.add(optionSubPanel, BorderLayout.CENTER);

    	optionSubPanel.add(new JLabel("Text Color"), "1, 1");
    	optionSubPanel.add(new JLabel("Background Color"), "1, 3");
    	optionSubPanel.add(new JLabel("Border Color"), "1, 5");
    	optionSubPanel.add(new JLabel("Transparency Color"), "1, 7");
    	    	
    	optionSubPanel.add(useTransparencyRadioButton, "3, 9, 7, 9, l, f");
    	optionSubPanel.add(useTransparentColorRadioButton, "3, 7, l, f");
    	
    	optionSubPanel.add(selectedTextColorPanel, "5, 1, f, f");
    	optionSubPanel.add(selectedBackgroundColorPanel, "5, 3, f, f");
    	optionSubPanel.add(selectedBorderColorPanel, "5, 5, f, f");
    	optionSubPanel.add(selectedTransparencyColorPanel, "5, 7, f, f");
    	
    	
    	optionSubPanel.add(changeTextColorButton, "7, 1");
    	optionSubPanel.add(changeBackgroundColorButton, "7, 3");
    	optionSubPanel.add(changeBorderColorButton, "7, 5");
    	optionSubPanel.add(changeTransparencyColorButton, "7, 7");
    	
    	ConfigManager configManager = WallGenerator.getConfigManager();
    	selectedTextColorPanel.setBackground(configManager.getColor("Text"));
    	selectedBackgroundColorPanel.setBackground(configManager.getColor("Background"));
    	selectedTransparencyColorPanel.setBackground(configManager.getColor("Transparency"));
    	selectedBorderColorPanel.setBackground(configManager.getColor("Border"));

    	transparencyRadioButtonGroup.add(useTransparencyRadioButton);
    	transparencyRadioButtonGroup.add(useTransparentColorRadioButton);
    	
    	useTransparentColorRadioButton.setSelected(true);
    	
        selectedBackgroundColorPanel.setBorder(BorderFactory.createEtchedBorder());
        selectedTransparencyColorPanel.setBorder(BorderFactory.createEtchedBorder());
        selectedTextColorPanel.setBorder(BorderFactory.createEtchedBorder());
        selectedBorderColorPanel.setBorder(BorderFactory.createEtchedBorder());

    	
    }
    
    private void setupProcessingPanel() {
    	final JSpinner.NumberEditor nearByEditor = new JSpinner.NumberEditor(nearByDarkeningSpinner, "0%");  
    	nearByDarkeningSpinner.setEditor(nearByEditor);
    	
    	final JSpinner.NumberEditor farAwayEditor = new JSpinner.NumberEditor(farAwayDarkeningSpinner, "0%");
    	farAwayDarkeningSpinner.setEditor(farAwayEditor);
    	
    	final JSpinner.NumberEditor softeningEditor = new JSpinner.NumberEditor(softeningSpinner, "0%");
    	softeningSpinner.setEditor(softeningEditor);
    	
    	farAwayEditor.getTextField().setInputVerifier(new PercentageSpinnerInputVerifier());
    	nearByEditor.getTextField().setInputVerifier(new PercentageSpinnerInputVerifier());    	
    	softeningEditor.getTextField().setInputVerifier(new PercentageSpinnerInputVerifier());
    	
    	final double[][] processingSubPanelLayoutDescriptor = new double[][] {
    			{3, TableLayout.PREFERRED, 3, TableLayout.PREFERRED, 3},
    			{10, TableLayout.PREFERRED, 3, TableLayout.PREFERRED, 3, 
    				TableLayout.PREFERRED, 10, TableLayout.PREFERRED, 3, TableLayout.PREFERRED, 
    				10, TableLayout.PREFERRED, 3, TableLayout.PREFERRED, 3, TableLayout.PREFERRED,
    				3, TableLayout.PREFERRED, TableLayout.FILL, 3}};
    	final TableLayout processingSubPanelLayout = new TableLayout(processingSubPanelLayoutDescriptor);
    	
    	final JPanel optionSubPanel = new JPanel(processingSubPanelLayout);
    	processingPanel = new JPanel(new BorderLayout());
    	processingPanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
    	
    	final DefaultComboBoxModel scalingMethodModel = (DefaultComboBoxModel)scalingMethodComboBox.getModel();
    	scalingMethodModel.addElement(new ComboBoxItemContainer("Nearest Neighbour", RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR));
    	scalingMethodModel.addElement(new ComboBoxItemContainer("Bilinear", RenderingHints.VALUE_INTERPOLATION_BILINEAR));
    	scalingMethodModel.addElement(new ComboBoxItemContainer("Bicubic", RenderingHints.VALUE_INTERPOLATION_BICUBIC));
    	scalingMethodModel.addElement(new ComboBoxItemContainer("Resample", "Resample"));
    	
    	final DefaultComboBoxModel unsharpMethodModel = (DefaultComboBoxModel)unsharpMaskComboBox.getModel();
    	unsharpMethodModel.addElement(new ComboBoxItemContainer("None", UnsharpenMask.None));
    	unsharpMethodModel.addElement(new ComboBoxItemContainer("Soft", UnsharpenMask.Soft));    	
    	unsharpMethodModel.addElement(new ComboBoxItemContainer("Normal", UnsharpenMask.Normal));
    	unsharpMethodModel.addElement(new ComboBoxItemContainer("Very Sharp", UnsharpenMask.VerySharp));
    	unsharpMethodModel.addElement(new ComboBoxItemContainer("Oversharpened", UnsharpenMask.Oversharpened));
    	
    	final JLabel titleLabel = new JLabel("Processing");
    	titleLabel.setHorizontalAlignment(SwingConstants.CENTER);
    	    	
    	processingPanel.add(titleLabel, BorderLayout.NORTH);
    	processingPanel.add(optionSubPanel, BorderLayout.CENTER);
    	
    	final JLabel darkeningLabel = getSectionLabel("Distance Darkening");
    	
    	optionSubPanel.add(darkeningLabel, "1, 1, 3, 1");
    	optionSubPanel.add(new JLabel("Near By Walls"), "1, 3");
    	optionSubPanel.add(nearByDarkeningSpinner, "3, 3");
    	optionSubPanel.add(new JLabel("Far Away Walls"), "1, 5");
    	optionSubPanel.add(farAwayDarkeningSpinner, "3, 5");
    	
    	optionSubPanel.add(getSectionLabel("Image Scaling"), "1, 7, 3, 7");
    	optionSubPanel.add(scalingMethodComboBox, "1, 9, 3, 9, f, f");
    	
    	optionSubPanel.add(getSectionLabel("Filters"), "1, 11, 3, 11");
    	optionSubPanel.add(new JLabel("Order"), "1, 13");
    	optionSubPanel.add(processOrderComboBox, "3, 13, f, f");
    	
    	optionSubPanel.add(new JLabel("Unsharp Mask"), "1, 15");
    	optionSubPanel.add(unsharpMaskComboBox, "3, 15, f, f");
    	
    	optionSubPanel.add(new JLabel("Soften"), "1, 17");
    	optionSubPanel.add(softeningSpinner, "3, 17");
    }

    private JLabel getSectionLabel(String label) {
    	final JLabel newLabel = new JLabel(label);
    	newLabel.setFont(newLabel.getFont().deriveFont(Font.ITALIC | Font.BOLD));
    	return newLabel;
    }
    
    private void setupResolutionPanel() {
        final Collection<String> availableResolutions = WallGenerator.getConfigManager().getAvailableResolutions();

        final JPanel optionSubPanel = new JPanel(new GridLayout(availableResolutions.size(), 1));
        resolutionPanel = new JPanel(new BorderLayout());
        resolutionPanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));

        final JLabel titleLabel = new JLabel("Resolution");
        titleLabel.setHorizontalAlignment(SwingConstants.CENTER);

        resolutionPanel.add(titleLabel, BorderLayout.NORTH);
        resolutionPanel.add(optionSubPanel, BorderLayout.CENTER);

        for(String resolution : availableResolutions) {
            final JRadioButton resolutionButton = new JRadioButton(resolution);
            resolutionButtonGroup.add(resolutionButton);
            resolutionButtons.add(resolutionButton);
            optionSubPanel.add(resolutionButton);
        }

        resolutionButtons.iterator().next().setSelected(true);
    }

    private void setupOutputFormatPanel() {
        final Collection<String> availableOutputFormats = WallGenerator.getConfigManager().getAvailableOutputFormats();

        int sizeToSet = availableOutputFormats.size();
        int blankItems = 0;
        if (sizeToSet < 6) {
        	sizeToSet = 6;
        	blankItems = sizeToSet - availableOutputFormats.size();
        }
        
        final JPanel optionSubPanel = new JPanel(new GridLayout(sizeToSet, 1));
        outputFormatPanel = new JPanel(new BorderLayout());
        outputFormatPanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));

        final JLabel titleLabel = new JLabel("Output Format");
        titleLabel.setHorizontalAlignment(SwingConstants.CENTER);

        outputFormatPanel.add(titleLabel, BorderLayout.NORTH);
        outputFormatPanel.add(optionSubPanel, BorderLayout.CENTER);

        for(String outputFormat : availableOutputFormats) {
            final JRadioButton outputFormatButton = new JRadioButton(outputFormat);
            outputFormatButtonGroup.add(outputFormatButton);
            outputFormatButtons.add(outputFormatButton);
            optionSubPanel.add(outputFormatButton);
        }
        
        for(int i = 0; i < blankItems; i++) {
        	optionSubPanel.add(new JLabel());
        }

        int index = 0;
        for(JRadioButton button : outputFormatButtons) {
        	if (index == 0) {
        		button.setSelected(true);
        	}
        	
        	if (button.getText().startsWith("Individual")) {
        		button.setEnabled(false);
        	}
        	
        	index++;
        }
    }

    private void setupOutputFileTypePanel() {
        final Collection<String> availableOutputFileTypes = WallGenerator.getConfigManager().getAvailableOutputFileTypes();

        int sizeToSet = availableOutputFileTypes.size();
        int blankItems = 0;
        if (sizeToSet < 6) {
        	sizeToSet = 6;
        	blankItems = sizeToSet - availableOutputFileTypes.size();
        }	
        
        final JPanel optionSubPanel = new JPanel(new GridLayout(sizeToSet, 1));
        outputFileTypePanel = new JPanel(new BorderLayout());
        outputFileTypePanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));

        final JLabel titleLabel = new JLabel("Output File Type");
        titleLabel.setHorizontalAlignment(SwingConstants.CENTER);

        outputFileTypePanel.add(titleLabel, BorderLayout.NORTH);
        outputFileTypePanel.add(optionSubPanel, BorderLayout.CENTER);

        for(String availableOutputFileType : availableOutputFileTypes) {
            final JRadioButton outputFileTypeButton = new JRadioButton(availableOutputFileType);
            outputFileTypeButtonGroup.add(outputFileTypeButton);
            outputFileTypeButtons.add(outputFileTypeButton);
            optionSubPanel.add(outputFileTypeButton);
        }
        for(int i = 0; i < blankItems; i++) {
        	optionSubPanel.add(new JLabel());
        }

        JRadioButton firstRadioButton = outputFileTypeButtons.iterator().next();
        
        firstRadioButton.setSelected(true);
       	setTransparencyEnabledState(firstRadioButton);
        
    }



    public void addChangeGenerationSettingsListener(ActionListener actionListener) {
        this.changeGenerationSettingsListeners.add(actionListener);
    }

    public void addPreviewWallsActionListener(ActionListener actionListener) {
        this.previewWallsActionListeners.add(actionListener);
    }

    public void addGenerateWallsActionListeners(ActionListener actionListener) {
        this.generateWallsActionListeners.add(actionListener);
    }

    private void fireChangeGenerationSettingsListeners(ActionEvent event) {
        for(ActionListener listener : this.changeGenerationSettingsListeners) {
        	log.debug("Invalidating Wall Cache...");
            listener.actionPerformed(event);
        }
    }

    private void firePreviewWallsActionListeners(ActionEvent event) {
        for(ActionListener listener : this.previewWallsActionListeners) {
            listener.actionPerformed(event);
        }
    }

    private void fireGenerateWallsActionListeners(ActionEvent event) {
        for(ActionListener listener : this.generateWallsActionListeners) {
            listener.actionPerformed(event);
        }
    }

    private void attachListeners() {
    	this.changeBackgroundColorButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				handleColorChange("Background", selectedBackgroundColorPanel.getBackground(), selectedBackgroundColorPanel);
			}
    	});

    	this.changeTextColorButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				handleColorChange("Text", selectedTextColorPanel.getBackground(), selectedTextColorPanel);				
			}
    	});

    	this.changeTransparencyColorButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				handleColorChange("Transparency", selectedTransparencyColorPanel.getBackground(), selectedTransparencyColorPanel);				
			}
    	});

    	this.changeBorderColorButton.addActionListener(new ActionListener() {
    		@Override
    		public void actionPerformed(ActionEvent e) {
    			handleColorChange("Border", selectedBorderColorPanel.getBackground(), selectedBorderColorPanel);
    		}
    	});
    	
        this.previewWallsButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                firePreviewWallsActionListeners(e);
            }
        });

        this.generateWallsButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                fireGenerateWallsActionListeners(e);
            }
        });
        
        final ActionListener outputTypeTransparencySupportListener = new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				final JRadioButton actionButton = (JRadioButton)e.getSource();

				if (actionButton.isSelected()) {
					setTransparencyEnabledState(actionButton);										
				}								
			}        	
        };
        
        for(JRadioButton outputFileTypeButton : outputFileTypeButtons) {
        	outputFileTypeButton.addActionListener(outputTypeTransparencySupportListener);
        }
        
        
        final ChangeListener invalidateGeneratedWallsChangeListener = new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				fireChangeGenerationSettingsListeners(new ActionEvent(e.getSource(), 0, null));
			}        	
        }; 
        
        final ActionListener invalidateGeneratedWallsActionListener = new ActionListener() {
        	@Override
        	public void actionPerformed(ActionEvent e) {
        		fireChangeGenerationSettingsListeners(e);
        	}
        };
        
        nearByDarkeningSpinner.addChangeListener(invalidateGeneratedWallsChangeListener);
        farAwayDarkeningSpinner.addChangeListener(invalidateGeneratedWallsChangeListener);
        softeningSpinner.addChangeListener(invalidateGeneratedWallsChangeListener);
        
        scalingMethodComboBox.addActionListener(invalidateGeneratedWallsActionListener);
        unsharpMaskComboBox.addActionListener(invalidateGeneratedWallsActionListener);
        processOrderComboBox.addActionListener(invalidateGeneratedWallsActionListener);
        
        final ChangeGenerationSettingActionListener changeListener = new ChangeGenerationSettingActionListener();

        final Collection<JRadioButton> generationSettingChangeButtons = new ArrayList<JRadioButton>();
        generationSettingChangeButtons.addAll(outputFileTypeButtons);
        generationSettingChangeButtons.addAll(outputFormatButtons);
        generationSettingChangeButtons.addAll(resolutionButtons);

        for(JRadioButton button : generationSettingChangeButtons) {
            button.addActionListener(changeListener);
        }
    }
    
    private void handleColorChange(String type, Color initialColor, JPanel targetPanel) {
    	Color newColor = JColorChooser.showDialog(this, "Select " + type + " Color", initialColor);
    	if (newColor != null) {
    		WallGenerator.getConfigManager().setColor(type, newColor);
    		targetPanel.setBackground(newColor);
    	}
    }

    private void setTransparencyEnabledState(JRadioButton outputFileTypeRadioButton) {
		final OutputFileConfig outputFileConfig = WallGenerator.getConfigManager().getOutputFileConfig(outputFileTypeRadioButton.getText());
		final boolean supportsTransparency = outputFileConfig.isSupportsTransparency();
		
		useTransparencyRadioButton.setEnabled(supportsTransparency);
		if (!supportsTransparency && useTransparencyRadioButton.isSelected()) {
			useTransparentColorRadioButton.setSelected(true);
		}
    }
    
    private final TableLayout setupLayout() {
        this.layoutDescriptor = new double[][] {
                {   3, TableLayout.PREFERRED, 
                	10, TableLayout.PREFERRED, 
                	10, TableLayout.PREFERRED, 
                	10, TableLayout.PREFERRED, 
                	10, TableLayout.PREFERRED, 
                	3, TableLayout.FILL, 
                	3, TableLayout.PREFERRED, 3},
                {3, TableLayout.PREFERRED, 3, TableLayout.PREFERRED, 3, TableLayout.PREFERRED, 3, TableLayout.PREFERRED, 3}
        };
        this.layout = new TableLayout(this.layoutDescriptor);
        return this.layout;
    }

    private final void setupComponents() {


        outputFileTypeButtonGroup.add(pngFileTypeRadio);
        outputFileTypeButtonGroup.add(gifFileTypeRadio);
        outputFileTypeButtonGroup.add(bmpFileTypeRadio);

        outputFileTypeButtons.add(pngFileTypeRadio);
        outputFileTypeButtons.add(gifFileTypeRadio);
        outputFileTypeButtons.add(bmpFileTypeRadio);
        
    }

    private final void placeComponents() {
        this.add(resolutionPanel, "1, 1, 1, 5, f, f");

        this.add(outputFormatPanel, "3, 1, 3, 5, f, f");

        this.add(outputFileTypePanel, "5, 1, 5, 5, f, f");
        
        this.add(processingPanel, "7, 1, 7, 5, f, f");
        
        this.add(colorPanel, "9, 1, 9, 5, f, f");
        
        this.add(buttonPanel, "1, 7, 13, 7, f, f");
    }

    public String getSelectedResolution() {
        for(JRadioButton button : resolutionButtons) {
            if (button.isSelected()) {
                return button.getText();
            }
        }
        return null;
    }

    public String getSelectedFileType() {
        for(JRadioButton button : outputFileTypeButtons) {
            if (button.isSelected()) {
                return button.getText();
            }
        }
        return null;
    }

    public String getSelectedOutputType() {
        for(JRadioButton button : outputFormatButtons) {
            if (button.isSelected()) {
                return button.getText();
            }
        }
        return null;
    }
    
    public double getNearWallDarkening() {
    	return (Double)nearByDarkeningSpinner.getValue();
    }
    
    public double getFarWallDarkening() {
    	return (Double)farAwayDarkeningSpinner.getValue();
    }

    public class ChangeGenerationSettingActionListener implements ActionListener {
        @Override
        public void actionPerformed(ActionEvent event) {
            fireChangeGenerationSettingsListeners(event);
        }
    }

	public boolean useTransparency() {
		return this.useTransparencyRadioButton.isSelected();
	}
	
	public PostProcessingConfig getPostProcessingConfig() {
		final PostProcessingConfig config = new PostProcessingConfig();
		
		config.farDarkening = ((Double)this.farAwayDarkeningSpinner.getValue()).floatValue();
		config.nearDarkening = ((Double)this.nearByDarkeningSpinner.getValue()).floatValue();
		config.scalingMethod = ((ComboBoxItemContainer)scalingMethodComboBox.getSelectedItem()).getItem();
		config.softening = ((Double)this.softeningSpinner.getValue()).floatValue();
		config.unsharpenMask = (UnsharpenMask)((ComboBoxItemContainer)unsharpMaskComboBox.getSelectedItem()).getItem();
		config.postProcessingOrder = this.processOrderComboBox.getSelectedIndex();
		
		return config;
	}	

	private class PercentageSpinnerInputVerifier extends InputVerifier {
		@Override
		public boolean verify(JComponent input) {
			final JTextField field = (JTextField)input;
			String textToVerify = field.getText().trim();
			
			String numberValue = null;
			if (textToVerify.endsWith("%")) {
				numberValue = textToVerify.substring(0, textToVerify.length() - 1);
			} else {
				numberValue = textToVerify;
			}
			
			try {
				int value = Integer.parseInt(numberValue);
				if (value >= 0 && value <= 100) {
					field.setText(value + "%");
					return true;
				} else {
					JOptionPane.showMessageDialog(OptionPanel.this, "Darkening value must be between 0% and 100%", "Invalid Darkening Value", JOptionPane.ERROR_MESSAGE);
				}
			} catch (NumberFormatException nfe) {
				JOptionPane.showMessageDialog(OptionPanel.this, "Invalid darkening value", "Invalid Darkening Value", JOptionPane.ERROR_MESSAGE);
			}
			
			return false;
		}
	}
	
    public class ComboBoxItemContainer {
    	private final String label;
    	private final Object item;
    	
    	public ComboBoxItemContainer(String label, Object item) {
    		this.label = label;
    		this.item = item;
    	}
    	
    	@Override
    	public String toString() {
    		return this.label;
    	}
    	
    	public Object getItem() {
    		return this.item;
    	}
    }
}
