/*
Copyright 2011 Daniel Morton. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice, this list of
      conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright notice, this list
      of conditions and the following disclaimer in the documentation and/or other materials
      provided with the distribution.

THIS SOFTWARE IS PROVIDED BY Daniel Morton ''AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those of the
authors and should not be interpreted as representing official policies, either expressed
or implied, of Daniel Morton.
*/

package ca.quadrilateral.wallgenerator;

import info.clearthought.layout.TableLayout;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.FileNotFoundException;
import java.io.IOException;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class WallGeneratorFrame extends JFrame {
	private static final Logger log = LoggerFactory.getLogger(WallGeneratorFrame.class);
    private static final long serialVersionUID = 1L;

    private Controller controller = null;

    public WallGeneratorFrame() {
        super("WallGenerator v0.9.2");
        this.setDefaultCloseOperation(EXIT_ON_CLOSE);
        this.setSize(1224, 550);

        final FileChooserPanel fileChooserPanel = new FileChooserPanel();
        final OptionPanel optionPanel = new OptionPanel();

        this.getContentPane().setLayout(new BorderLayout());
        this.getContentPane().add(fileChooserPanel, BorderLayout.NORTH);
        this.getContentPane().add(optionPanel, BorderLayout.CENTER);

        double[][] buttonPanelLayoutDescriptor = new double[][] {{5, TableLayout.FILL, TableLayout.PREFERRED, 5, TableLayout.PREFERRED, 5},{5, TableLayout.PREFERRED, 5}};
        TableLayout buttonPanelLayout = new TableLayout(buttonPanelLayoutDescriptor);
        JPanel buttonPanel = new JPanel(buttonPanelLayout);

        JButton exitButton = new JButton("Exit");
        JButton reloadConfigButton = new JButton("Reload Config");
        buttonPanel.add(new JLabel("Wall Generator - By Daniel Morton (djmorton@quadrilateral.ca) - www.quadrilateral.ca/wallgen"), "1, 1, l, f");
        buttonPanel.add(reloadConfigButton, "2, 1");
        buttonPanel.add(exitButton, "4, 1");

        exitButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent event) {
                WallGeneratorFrame.this.setVisible(false);
                log.info("Wall Generation Application Terminated");
                System.exit(0);
            }
        });

        reloadConfigButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                try {
                    WallGenerator.getConfigManager().reloadConfig();
                    controller.clearGeneratedImages();
                } catch (Exception ex) {
                    JOptionPane.showMessageDialog(WallGeneratorFrame.this, "Error reloading config.xml file", "Config Error", JOptionPane.ERROR_MESSAGE);
                }
            }
        });

        this.getContentPane().add(buttonPanel, BorderLayout.SOUTH);


        this.controller = new Controller(fileChooserPanel, optionPanel, WallGenerator.getConfigManager());

        optionPanel.addPreviewWallsActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                try {
                    final BufferedImage previewImage = controller.generatePreview();
                    PreviewDialog previewDialog = new PreviewDialog(previewImage, WallGeneratorFrame.this);
                    previewDialog.setVisible(true);
                } catch (FileNotFoundException fnfe) {
                	log.error("Error generating preview", fnfe);
                	JOptionPane.showMessageDialog(WallGeneratorFrame.this, "Source image could not be found!", "Source Image Error", JOptionPane.ERROR_MESSAGE);
                } catch (IOException ioe) {
                	log.error("Error generating preview", ioe);
                	JOptionPane.showMessageDialog(WallGeneratorFrame.this, "IO error generating images.  Check file permissions", "IO Error", JOptionPane.ERROR_MESSAGE);
                } catch (Exception ex) {
                	log.error("Error generating preview", ex);
                }
            }
        });

        optionPanel.addGenerateWallsActionListeners(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                try {
                	if (StringUtils.isBlank(fileChooserPanel.outputFileNameBase())) {
                		JOptionPane.showMessageDialog(WallGeneratorFrame.this, "No base output file name specified!", "Target Image Error", JOptionPane.ERROR_MESSAGE);
                	} else {
                		controller.generateOutput();
                		JOptionPane.showMessageDialog(WallGeneratorFrame.this, "Generation Completed!", "Complete", JOptionPane.INFORMATION_MESSAGE);
                	}
                } catch (FileNotFoundException fnfe) {
                	log.error("Error generating output", fnfe);
                	JOptionPane.showMessageDialog(WallGeneratorFrame.this, "Source image could not be found!", "Source Image Error", JOptionPane.ERROR_MESSAGE);
                } catch (IOException ioe) {
                	log.error("Error generating output", ioe);
                	JOptionPane.showMessageDialog(WallGeneratorFrame.this, "IO error generating images.  Check file permissions", "IO Error", JOptionPane.ERROR_MESSAGE);                	
                } catch (Exception ex) {
                	log.error("Error generating output", ex);
                	JOptionPane.showMessageDialog(WallGeneratorFrame.this, "An unanticipated error has ocurred.  See log file for details", "Error", JOptionPane.ERROR_MESSAGE);
                }
            }
        });

    }

}
